﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;

namespace RobotCtrl
{
    public class TrackPylons : Track
    {
        private readonly Radar radar;
        private Track runningTrack;
        private Track nextTrack;
        /// <summary>
        /// This is an endless Track
        /// </summary>
        public override bool Done { get { return false; } }

        public override double ResidualLength { get { return 100.0; /* never ending track */ } }

        public TrackPylons(double maxSpeed, double maxAcceleration, Radar radar)
        {
            if (radar == null)
                throw new ArgumentNullException("radar");

            this.radar = radar;

            reverse = false;
            this.nominalSpeed = Math.Abs(maxSpeed);
            this.acceleration = maxAcceleration;
            this.runningTrack = new TrackForcedLine(2, maxSpeed, maxAcceleration);
        }

        public override void IncrementalStep(double timeInterval, double newVelocity, out double leftSpeed, out double rightSpeed)
        {
            currentVelocity = newVelocity;

            double distance = radar.Distance;
            if (runningTrack != null && nextTrack != null &&
                runningTrack.Done)
            {
                runningTrack = nextTrack;
                nextTrack = null;
            }
            else if ((runningTrack is TrackArcLeft || runningTrack is TrackForcedLine) && !runningTrack.Done)
            {
                // ensure the arc/"first line" are allways finished
            }
            else if (distance > 1.2)
            {
                if (runningTrack == null || !(runningTrack is TrackLine))
                {
                    runningTrack = new TrackLine(10, newVelocity, acceleration);
                }
            }
            else
            {
                if (runningTrack == null || !(runningTrack is TrackArcLeft))
                {
                    double turnAngle = 180.0 - (0.6-distance) * 10.0;
                    runningTrack = new TrackArcLeft((distance + 0.6) / 2, turnAngle, newVelocity, acceleration);

                    // Vom Pylon wegfahren, damit nicht gleich mit dem nächsten
                    // Halbkreis begonnen wird.
                    nextTrack = new TrackForcedLine(.5, newVelocity, acceleration);
                }
            }

            runningTrack.IncrementalStep(timeInterval, newVelocity, out leftSpeed, out rightSpeed);
            this.doStep(timeInterval);
        }

        /// <summary>
        /// Linie, die zwingend zu Ende gefahren werden muss (z.B. am 1. Pylon vorbei fahren oder nach einem Halbkreis)
        /// </summary>
        private class TrackForcedLine : TrackLine
        {
            public TrackForcedLine(double length, double speed, double acceleration)
                : base(length, speed, acceleration) { }
        }
    }
}
