using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using RobotCtrl;
using RobotView;

namespace Test_Console_CE
{
    public partial class TestWorld : Form
    {
        private Drive drive;

        public TestWorld()
        {
            InitializeComponent();

            // Roboter-Reset-Parameter
            this.resetXUpDown.Value = 0;
            this.resetYUpDown.Value = 0;
            this.resetAngleUpDown.Value = (decimal)0;
            this.speedUpDown.Value = 1000;

            // Roboter kreieren und konfigurieren
            Robot robot = new Robot(RunMode.VIRTUAL);
            drive = robot.Drive;
            drive.Power = true;
            reset();

            // Roboter ins World-Zentrum hineinsetzen
            World.Robots.Add(robot);

            // Obstacles kreieren und in World einfgen
            RectangleF pylonArea1 = new RectangleF(0.875f, 0.375f, 0.25f, 0.25f); 
            ObstacleMap pylon1 = new ObstacleMap("Resources\\Pylon.bmp", pylonArea1);
            World.Obstacles.Add(pylon1);
            RectangleF pylonArea2 = new RectangleF(2.875f, 0.375f, 0.25f, 0.25f);
            ObstacleMap pylon2 = new ObstacleMap("Resources\\Pylon.bmp", pylonArea2);
            World.Obstacles.Add(pylon2);

            RectangleF obstacle1Area = new RectangleF(0f, -3f, 3f, 3f);
            ObstacleMap obstacle1 = new ObstacleMap("Resources\\Obstacle1.bmp", obstacle1Area);
            World.Obstacles.Add(obstacle1);

            // Views ffnen
            DriveView driveView = new DriveView(drive);
            this.Controls.Add(driveView);
            WorldForm worldForm = new WorldForm(World.Area);
            ConsoleForm consoleForm = new ConsoleForm(robot.Console);
#if !WindowsCE
			this.StartPosition = FormStartPosition.Manual;
			worldForm.StartPosition = FormStartPosition.Manual;
			consoleForm.StartPosition = FormStartPosition.Manual;
#endif
            this.Location = new Point(0, 0);
            worldForm.Height = Screen.PrimaryScreen.WorkingArea.Height; //BULLSHIT!!
            worldForm.ClientSize = new Size(worldForm.ClientSize.Height, worldForm.ClientSize.Height);
            worldForm.Show();
            worldForm.Location = new Point(Screen.PrimaryScreen.WorkingArea.Width - worldForm.Width, 0);

            consoleForm.Location = new Point(worldForm.Left, 0);
            consoleForm.TopMost = true;
            consoleForm.Text = "World-Test";
            consoleForm.ControlBox = false;
            //consoleForm.Show();

            // GUI-Logik starten
            System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
            timer.Interval = 200;
            timer.Enabled = true;
            timer.Tick += new EventHandler(timer_Tick);
        }

        void reset()
		{
			drive.Stop();
			drive.Position.X = (double)resetXUpDown.Value / 1000.0;
            drive.Position.Y = (double)resetYUpDown.Value / 1000.0;
            drive.Position.Angle =  (double)resetAngleUpDown.Value;
		}



        void timer_Tick(object sender, EventArgs e)
        {
            linStartButton.Enabled = drive.Done;
            arcStartButton.Enabled = drive.Done;
            turnStartButton.Enabled = drive.Done;
        }

        void Form1_Closing(object sender, CancelEventArgs e)
        {
            foreach (Robot r in World.Robots)
            {
                r.Drive.Close();
            }
        }

        void stopButton_Click(object sender, EventArgs e)
        {
            //drive.Stop();
            drive.Halt();
        }

        void resetButton_Click(object sender, EventArgs e)
        {
            reset();
        }

        void lineNegButton_Click(object sender, EventArgs e)
        {
            linLengthUpDown.Value = -linLengthUpDown.Value;
        }

        void arcNegButton_Click(object sender, EventArgs e)
        {
            arcAngleUpDown.Value = -arcAngleUpDown.Value;
        }

        void turnNegButton_Click(object sender, EventArgs e)
        {
            turnAngleUpDown.Value = -turnAngleUpDown.Value;
        }

        void linStartButton_Click(object sender, EventArgs e)
        {
            double length = (double)linLengthUpDown.Value / 1000.0;
            double speed = (double)speedUpDown.Value / 1000.0;
            double acceleration = (double)accelerationUpDown.Value / 1000.0;
            drive.RunLine(length, speed, acceleration);
        }

        void arcStartButton_Click(object sender, EventArgs e)
        {
            double radius = (double)arcRadiusUpDown.Value / 1000.0;
            double angle = (double)arcAngleUpDown.Value;
            double speed = (double)speedUpDown.Value / 1000.0;
            double acceleration = (double)accelerationUpDown.Value / 1000.0;
            if (arcLeftRadioButton.Checked)
                drive.RunArcLeft(radius, angle, speed, acceleration);
            else
                drive.RunArcRight(radius, angle, speed, acceleration);
        }

        void turnStartButton_Click(object sender, EventArgs e)
        {
            double angle = (double)turnAngleUpDown.Value;
            double speed = (double)speedUpDown.Value / 1000.0;
            double acceleration = (double)accelerationUpDown.Value / 1000.0;
            drive.RunTurn(angle, speed, acceleration);
        }
    }
}